import importlib.util
import json
from typing import Any
from pathlib import Path
from pydantic import BaseModel

class PluginLoader:
    def __init__(self, plugins_dir: Path):
        self.plugins_dir = plugins_dir

    def load_plugins(self):
        for file in self.plugins_dir.iterdir():
            if file.suffix == ".py":
                spec = importlib.util.spec_from_file_location(file.stem, file)
                if not spec or not spec.loader:
                    continue
                
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)


def is_serializable(obj: Any):
    if isinstance(obj, BaseModel):
        return True
    
    try:
        json.dumps(obj)
        return True
    except (TypeError, ValueError):
        return False
